/**
 * @author Susanne Stadler, DEMO CHI12
 * 
 * Gives the user the ability let the Smart phone feel the surface with vibration and accelerometer
 * 1) Accelerometer is set on SENSOR_DELAY_FASTEST
 * 2) Vibrator vibrates 4 seconds, during vibration accelerometer data will be added to accdata.
 *    Followed by sending data via HTTPREQUEST to server. 
 *    The result got form the HTTPREQUEST is either the classified label of the surface or 
 *    "<!--" if something goes wrong 
 *    For better readability we delete the the first part "chi_" from the label with removePrefix(String line)
 *    The label part "chi_" is needed for the functionality to reset data to demo_surfaces
 */

package bzzzt.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Vibrator;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class TestActivity extends Activity implements SensorEventListener {

	private static final String TAG = "TestActivity";

	private String url_svm_test = "http://192.168.2.41/cgi-bin/chi_svm_test.py";

	private TextView datax, datay, dataz; // Shows accelerometer data
	private TextView result; // Shows label requested from server
	private String accdata; // Holds data to send to server
	private boolean isVibrating = false; // Proves is Smart phone vibrating
	private SensorManager sensorManager; // Needed service accelerometer
	private Vibrator vib; // Needed service vibrator

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.test);
		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		datax = (TextView) findViewById(R.id.dataX);
		datay = (TextView) findViewById(R.id.dataY);
		dataz = (TextView) findViewById(R.id.dataZ);
		result = (TextView) findViewById(R.id.result);
		accdata = "";

		sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_FASTEST);

		vib = (Vibrator) getSystemService(VIBRATOR_SERVICE);

		Button send_button = (Button) findViewById(R.id.Button_test);
		send_button.setOnClickListener(btnTest_listener);

	}

	public String removePrefix(String line) {
		if (line.startsWith("chi_")) {
			line = line.substring(4);
		}
		return line;
	}

	private OnClickListener btnTest_listener = new OnClickListener() {
		public void onClick(View v) {
			// set label to processing while waiting to response from server
			result.setText("processing");
			result.postInvalidate();

			isVibrating = true;
			
            //vibrate 4 seconds
			vib.vibrate(4000);
			Handler handler = new Handler();
			handler.postDelayed(new Runnable() {
				public void run() {
					isVibrating = false;
					sendRequest(accdata);
					accdata = "";
				}
			}, 4002);
		}
	};

	public void sendRequest(String data) {

		HttpClient httpclient = new DefaultHttpClient();

		HttpPost httppost = new HttpPost(url_svm_test);
		
		List<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair("label", "chi_test"));
		params.add(new BasicNameValuePair("filename", "chi_test"
				+ System.currentTimeMillis() + ".txt"));
		params.add(new BasicNameValuePair("data", data));

		try {
			httppost.setEntity(new UrlEncodedFormEntity(params));

			HttpResponse response = httpclient.execute(httppost);

			InputStream is = response.getEntity().getContent();
			InputStreamReader ir = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(ir);
			String read = br.readLine();
			StringBuffer sb = new StringBuffer();

			while (read != null) {
				sb.append(removePrefix(read.trim()));
				read = br.readLine();
			}
			if (sb.toString().startsWith("<!--:")) {
				result.setText("Error happens! Try again.");
			} else {
				result.setText(removePrefix(sb.toString()));
			}
			result.postInvalidate();

		} catch (UnsupportedEncodingException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		} catch (ClientProtocolException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		} catch (IOException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		}
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	protected void onResume() {
		super.onResume();
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_FASTEST);
	}

	@Override
	protected void onPause() {
		// unregister listener
		sensorManager.unregisterListener(this);
		super.onStop();
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
			float[] values = event.values;
			// Movement
			float x = values[0];
			float y = values[1];
			float z = values[2];

			String d_x = String.valueOf(x);
			String d_y = String.valueOf(y);
			String d_z = String.valueOf(z);

			datax.setText(d_x);
			datay.setText(d_y);
			dataz.setText(d_z);

			datax.postInvalidate();
			datay.postInvalidate();
			dataz.postInvalidate();

			if (isVibrating) {
				long time = event.timestamp;
				accdata = accdata + time + " x:" + d_x + " y:" + d_y + " z:"
						+ d_z + "\n";
			}
		}
	}

}
