/**
 * @author Susanne Stadler, DEMO CHI12
 * 
 * Gives the user the ability to learn the system new surfaces
 * 1) User can enter the label of the new surface and send the data 
 *    via an HTTPREQUEST to the server.
 *    The response from the server is the filename under which the before
 *    sended data is saved in the training data set. 
 * 2) For better usability we have added the functionality to show a list.
 *    For better readability we delete the first part "chi_" of the label with removePrefix(String line)
 *    The label part "chi_" is needed for the functionality to reset data to demo_surfaces
 */

package bzzzt.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Vibrator;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

public class TrainActivity extends Activity implements SensorEventListener {

	private static final String TAG = "TrainActivity";
	private boolean train_default = false;

	private String url_svm_train = "http://192.168.2.41/cgi-bin/chi_svm_train.py";
	private String url_svm_list = "http://192.168.2.41/cgi-bin/chi_list_data.py";

	private TextView result; // Shows if data is added or not
	private EditText filenameMedium; // Textfield to enter label for surface
	
	private SensorManager sensorManager;// Needed service accelerometer
	private Vibrator vib;// Needed service vibrator
	
	private boolean isVibrating = false; // Proves is Smart phone vibrating
	private String accdata; // Holds data to send to server
	private String surfacename; //Entered name of surface = label of surface for server

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.train);
		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_FASTEST);

		vib = (Vibrator) getSystemService(VIBRATOR_SERVICE);

		Button train_button = (Button) findViewById(R.id.Button_train);
		train_button.setOnClickListener(btnTrain_listener);

		Button list_button = (Button) findViewById(R.id.Button_requestList);
		list_button.setOnClickListener(btnList_listener);

		result = (TextView) findViewById(R.id.add_result);
		filenameMedium = (EditText) findViewById(R.id.filenameMedium);
		accdata = "";
	}

	private OnClickListener btnTrain_listener = new OnClickListener() {
		public void onClick(View v) {
			
			if(train_default){
				surfacename = "demo_"+filenameMedium.getText().toString();
			}else{
				surfacename = filenameMedium.getText().toString();
			}
			result.setText("processing");
			result.postInvalidate();

			vib.vibrate(4000);
			isVibrating = true;

			Handler handler = new Handler();
			handler.postDelayed(new Runnable() {
				public void run() {
					isVibrating = false;
					sendRequest(url_svm_train, surfacename, accdata);
					accdata = "";
				}
			}, 4002);
		}
	};

	private OnClickListener btnList_listener = new OnClickListener() {
		public void onClick(View v) {
			sendRequest(url_svm_list, null, null);
		}
	};

	public void sendRequest(String url, String label, String data) {
		HttpClient httpclient = new DefaultHttpClient();

		HttpPost httppost = null;

		try {
			if (url.equalsIgnoreCase(url_svm_train)) {
				httppost = new HttpPost(url_svm_train);

				List<NameValuePair> params = new ArrayList<NameValuePair>();
				params.add(new BasicNameValuePair("label", label));
				params.add(new BasicNameValuePair("filename", label
						+ System.currentTimeMillis() + ".txt"));
				params.add(new BasicNameValuePair("data", data));

				httppost.setEntity(new UrlEncodedFormEntity(params));
			}

			if (url.equalsIgnoreCase(url_svm_list)) {
				httppost = new HttpPost(url_svm_list);
			}

			HttpResponse response = httpclient.execute(httppost);

			InputStream is = response.getEntity().getContent();
			InputStreamReader ir = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(ir);
			String read = br.readLine();
			StringBuffer sb = new StringBuffer();

			if (url.equalsIgnoreCase(url_svm_train)) {
				while (read != null) {
					sb.append(read);
					read = br.readLine();
				}

				if (sb.toString().startsWith("<!--:")) {
					result.setText("Error happens! Try again.");
				} else {
					result.setText(removePrefix(sb.toString()));
				}

				result.postInvalidate();
			}

			if (url.equalsIgnoreCase(url_svm_list)) {
				LinearLayout layout = (LinearLayout) findViewById(R.id.linearLayout1);
				layout.removeAllViews();
				while (read != null) {
					sb.append(read);
					TextView textView = new TextView(this);
					textView.setText(removePrefix(read.trim()));
					layout.addView(textView);
					read = br.readLine();
				}
			}
		} catch (UnsupportedEncodingException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		} catch (ClientProtocolException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		} catch (IOException e) {
			Log.e(TAG, e.getMessage() + " in sendRequest");
		}
	}

	public String removePrefix(String line) {
		if (line.startsWith("chi_")) {
			line = line.substring(4);
		}
		return line;
	}

	protected void onResume() {
		super.onResume();
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_FASTEST);
	}

	@Override
	protected void onPause() {
		// unregister listener
		sensorManager.unregisterListener(this);
		super.onStop();
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
			float[] values = event.values;
			// Accelerometer data
			float x = values[0];
			float y = values[1];
			float z = values[2];

			String d_x = String.valueOf(x);
			String d_y = String.valueOf(y);
			String d_z = String.valueOf(z);

			if (isVibrating) {
				long time = event.timestamp;
				accdata = accdata + time + " x:" + d_x + " y:" + d_y + " z:"
						+ d_z + "\n";
			}
		}

	}

}
